@@ -41,20 +41,25 @@ DAEMON_CHECK_PERIOD = 25
 
 stdin_queue: queue.Queue[str] = queue.Queue()
 
+
 def read_stdin():
+    # Do not access C# objects from separate threads
     while True:
         line = sys.stdin.readline().strip()
         if line.startswith(COMMAND_CHAR):
             stdin_queue.put(line)
 
+
 # Create a new thread for reading from stdin
 t = threading.Thread(target=read_stdin, daemon=True)
 t.start()
 
-class AnimoTrainerController:
+
+class AnimoTrainingController:
     def __init__(self, session: AnimoTrainingSession):
         self.session = session
         run_options = session.run_options
+        self.env_manager = SimpleEnvManager(AnimoEnv(session), EnvironmentParametersChannel())
 
         self.trainer_factory = TrainerFactory(
             trainer_config=run_options.behaviors,
@@ -62,19 +67,16 @@ class AnimoTrainerController:
             train_model=not run_options.checkpoint_settings.inference,
             load_model=run_options.checkpoint_settings.resume,
             seed=run_options.env_settings.seed,
-            param_manager=EnvironmentParameterManager(), # Not used
-            init_path=run_options.checkpoint_settings.maybe_init_path, #type:ignore
+            param_manager=EnvironmentParameterManager(),  # Not used
+            init_path=run_options.checkpoint_settings.maybe_init_path,  # type:ignore
             multi_gpu=False,
         )
 
-        animo_env = AnimoEnv(session)
-        self.env_manager = SimpleEnvManager(animo_env, EnvironmentParametersChannel())
-
         self.trainers: Dict[str, RLTrainer] = {}
-        self.brain_name_to_identifier: Dict[str, Set] = defaultdict(set) #type:ignore
+        self.brain_name_to_identifier: Dict[str, Set] = defaultdict(set)  # type:ignore
         self.output_path = run_options.checkpoint_settings.write_path
         self.run_id = run_options.checkpoint_settings.run_id
-        self.train_model = not run_options.checkpoint_settings.inference,
+        self.train_model = not run_options.checkpoint_settings.inference
         self.ghost_controller = self.trainer_factory.ghost_controller
         self.registered_behavior_ids: Set[str] = set()
         self.trainer_threads: List[threading.Thread] = []
@@ -89,10 +91,9 @@ class AnimoTrainerController:
         else:
             logging_util.set_log_level(logging_util.INFO)
         np.random.seed(run_options.env_settings.seed)
-        torch_utils.torch.manual_seed(run_options.env_settings.seed) #type: ignore
+        torch_utils.torch.manual_seed(run_options.env_settings.seed)  # type: ignore
         self.rank = get_rank()
 
-
     @timed
     def _save_models(self):
         """
@@ -108,10 +109,10 @@ class AnimoTrainerController:
         self.logger.debug("Saved Model")
 
     @staticmethod
-    def _create_output_path(output_path): #type: ignore
+    def _create_output_path(output_path):  # type: ignore
         try:
-            if not os.path.exists(output_path): #type: ignore
-                os.makedirs(output_path) #type: ignore
+            if not os.path.exists(output_path):  # type: ignore
+                os.makedirs(output_path)  # type: ignore
         except Exception:
             raise UnityEnvironmentException(
                 f"The folder {output_path} containing the "
@@ -128,7 +129,7 @@ class AnimoTrainerController:
             A Data structure corresponding to the initial reset state of the
             environment.
         """
-        env_manager.reset() #type: ignore
+        env_manager.reset()  # type: ignore
         # Register any new behavior ids that were generated on the reset.
         self._register_new_behaviors(env_manager, env_manager.first_step_infos)
 
@@ -184,7 +185,7 @@ class AnimoTrainerController:
         )
         env_manager.set_agent_manager(name_behavior_id, agent_manager)
         env_manager.set_policy(name_behavior_id, policy)
-        self.brain_name_to_identifier[brain_name].add(name_behavior_id) #type: ignore
+        self.brain_name_to_identifier[brain_name].add(name_behavior_id)  # type: ignore
 
         trainer.publish_policy_queue(agent_manager.policy_queue)
         trainer.subscribe_trajectory_queue(agent_manager.trajectory_queue)
@@ -201,12 +202,13 @@ class AnimoTrainerController:
 
     @timed
     def train(self) -> None:
-        self._create_output_path(self.output_path) #type: ignore
+        self._create_output_path(self.output_path)  # type: ignore
         try:
             # Initial reset
             self._reset_env(self.env_manager)
             while self._not_done_training():
-                self.advance(self.env_manager)
+                with self.session.lock:
+                    self.advance(self.env_manager)
             # Stop advancing trainers
             self.join_threads()
         except (
@@ -229,22 +231,17 @@ class AnimoTrainerController:
             if self.train_model:
                 self._save_models()
 
-    def end_trainer_episodes(self) -> None:
-        # Reward buffers reset takes place only for curriculum learning
-        # else no reset.
-        for trainer in self.trainers.values():
-            trainer.end_episode()
-
     @timed
     def advance(self, env_manager: EnvManager) -> int:
-        #Check if interrupt training was passed through stdin
+        # Check if interrupt training was passed through stdin
         while not stdin_queue.empty():
             if stdin_queue.get() == INTERRUPTION_COMMAND:
                 raise KeyboardInterrupt
 
-        #Check if daemon process is dead
+        # Check if daemon process is dead
         self.step += 1
-        if self.session.should_check_daemon and self.step % DAEMON_CHECK_PERIOD == 0 and not psutil.pid_exists(self.session.daemon_pid):
+        if self.session.should_check_daemon and self.step % DAEMON_CHECK_PERIOD == 0 and not psutil.pid_exists(
+            self.session.daemon_pid):
             raise KeyboardInterrupt
 
         # Get steps
@@ -289,11 +286,11 @@ class AnimoTrainerController:
             except Exception:
                 pass
 
-        with hierarchical_timer("trainer_threads") as main_timer_node: #type: ignore
+        with hierarchical_timer("trainer_threads") as main_timer_node:  # type: ignore
             for trainer_thread in self.trainer_threads:
                 thread_timer_stack = get_timer_stack_for_thread(trainer_thread)
                 if thread_timer_stack:
-                    main_timer_node.merge( #type: ignore
+                    main_timer_node.merge(  # type: ignore
                         thread_timer_stack.root,
                         root_name="thread_root",
                         is_parallel=True,
@@ -301,6 +298,7 @@ class AnimoTrainerController:
                     merge_gauges(thread_timer_stack.gauges)
 
     def trainer_update_func(self, trainer: Trainer) -> None:
+        # Do not access C# objects from separate threads
         while not self.kill_trainers:
             with hierarchical_timer("trainer_advance"):
                 trainer.advance()
