@@ -83,7 +83,7 @@ def main():
             clr.AddReference(dll_path)
 
     from animo_trainer.animo_training_session import AnimoTrainingSession
-    from animo_trainer.animo_trainer_controller import AnimoTrainerController
+    from animo_trainer.animo_trainer_controller import AnimoTrainingController
     from animo_trainer.animo_stats_writers import register_animo_stats_writers
     from mlagents.plugins.trainer_type import register_trainer_plugins
     register_trainer_plugins()
@@ -96,7 +96,7 @@ def main():
 
     register_animo_stats_writers(training_session_data.run_options)
 
-    animo_trainer_controller = AnimoTrainerController(training_session_data)
+    animo_trainer_controller = AnimoTrainingController(training_session_data)
     print(f"Animo-Learn::Started")
     animo_trainer_controller.train()
     print(f"Animo-Learn::Stopped")
